<?php
/**
 * Main Plugin
 *
 * @package   dorzki\WordPress\TableOfContents
 * @author    Dor Zuberi <admin@dorzki.io>
 * @copyright 2023 dorzki
 * @version   1.0.0
 */

namespace dorzki\WordPress\TableOfContents;

/**
 * Class Plugin
 *
 * @package dorzki\WordPress\TableOfContents
 */
class Plugin {
	/** @var int */
	private int $counter = 0;
	/** @var string[] */
	private array $headings = [];


	/**
	 * Plugin constructor
	 */
	public function __construct() {
		add_action( 'init', [ $this, 'register_textdomain' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'load_assets' ] );

		add_filter( 'the_content', [ $this, 'build_toc' ] );

		add_shortcode( 'disable_toc', '__return_false' );
	}


	/**
	 * Register the plugin's text domain
	 */
	public function register_textdomain() {
		load_plugin_textdomain( 'dorzki-wp-toc' );
	}

	/**
	 * Load plugin CSS and JS assets
	 */
	public function load_assets() {
		wp_enqueue_style( 'toc_styles', DRZK_TOC_URL . 'assets/styles.css', [], '1.0.0' );

		wp_enqueue_script( 'toc_scripts', DRZK_TOC_URL . 'assets/scripts.js', [ 'jquery' ], '1.0.0', true );
	}


	/**
	 * Extracts and builds TOC
	 *
	 * @param string $content the post content
	 *
	 * @return string
	 */
	public function build_toc( string $content ): string {
		if ( $this->is_disabled( $content ) ) {
			return $content;
		}

		$content = preg_replace_callback( '/<(h2)>(.*?)<\/h2>/i', [ $this, 'extract_headings' ], $content );

		$toc = $this->generate_toc();

		return $toc . $content;
	}

	/**
	 * Adds title id number and saves the heading
	 *
	 * @param array $matches regex match array
	 *
	 * @return string
	 */
	public function extract_headings( array $matches ): string {
		$tag = sprintf( '<%1$s id="title_%2$s">%3$s</%1$s>', $matches[1], $this->counter, $matches[2] );

		++$this->counter;
		$this->headings[] = $matches[2];

		return $tag;
	}

	/**
	 * Generates a table of contents from the headings
	 *
	 * @return string
	 */
	public function generate_toc(): string {
		if ( empty( $this->headings ) ) {
			return '';
		}

		$html  = '<div class="post_toc">';
		$html .= '  <strong>' . esc_html__( 'Table of Contents', 'wp-toc' ) . '</strong>';
		$html .= '  <ol>';

		foreach ( $this->headings as $id => $text ) {
			$html .= "<li><a href='#title_{$id}'>{$text}</a></li>";
		}

		$html .= '  </ol>';
		$html .= '</div>';

		return $html;
	}

	/**
	 * Check if the user don't want TOC on this page
	 *
	 * @param string $content the post content
	 *
	 * @return boolean
	 */
	private function is_disabled( string $content ): bool {
		return ( false !== strpos( $content, '[disable_toc]' ) );
	}
}

// Init class.
new Plugin();
